==> clickhouse_tools_abm-1.1.51477.tar.gz: clickhouse_tools_abm-1.1.51477/clickhouse_tools/clickhouse_interface.py <==
import logging
from collections import defaultdict
from dataclasses import dataclass
from typing import List, Dict, Type, Union, Optional
from async_tools import AsyncOnStart
from dict_caster.extras import to_list

from .elements.aggregation_function import AggregationFunction
from .elements.filter_by import Filter
from .elements.order import Order
from .elements.column import Column
from .exceptions import WrongArgumentsException
from .elements.table import Table, ALL_COLUMNS
from .elements.output_format import OutputFormat
from .clickhouse_connector import ClickHouseConnector


logger = logging.getLogger(__file__)


class ClickHouseInterface(AsyncOnStart):
    @dataclass
    class Context:
        clickhouse_connector: ClickHouseConnector

    def __init__(self, context: Context):
        self.context = context
        self._name_to_entity: Dict[str, Type[Table]] = {}

    async def _on_start(self) -> None:
        for table in self._name_to_entity.values():
            await self.context.clickhouse_connector.create_table(table)

    def register_entity(self, entity: Type[Table]) -> None:
        self._name_to_entity[entity.__table_name__] = entity
        logger.info(f'{entity} registered')

    def get_entity_by_name(self, entity_name: str) -> Type[Table]:
        if entity := self._name_to_entity.get(entity_name):
            return entity
        raise WrongArgumentsException(f'Unknown entity: {entity_name}')

    async def get(self,
                  entity_name: str,
                  *,
                  columns: Optional[List[str]] = ALL_COLUMNS,
                  filter_by: Optional[List[dict]] = None,
                  search_by: Optional[List[dict]] = None,
                  order_by: Optional[List[dict]] = None,
                  group_by: Optional[List[str]] = None,
                  offset: int = 0,
                  limit: int = 100,
                  output_format: str = OutputFormat.Dict.value,
                  aggregation_functions: Optional[List[dict]] = None
                  ) -> list:
        entity = self.get_entity_by_name(entity_name)

        columns: List[Column] = entity.columns_from_str(columns)
        filter_by: List[Filter] = entity.filters_from_str(filter_by)
        search_by: List[Filter] = entity.filters_from_str(search_by)
        order_by: List[Order] = entity.orders_from_str(order_by)
        group_by: List[Column] = entity.groups_from_str(group_by)
        output_format: OutputFormat = entity.output_format_from_str(output_format)
        aggregation_functions: List[AggregationFunction] = entity.aggregation_functions_from_str(aggregation_functions)

        result = await self.context.clickhouse_connector.select(
            entity, columns=columns, filter_by=filter_by, search_by=search_by, order_by=order_by, group_by=group_by,
            offset=offset, limit=limit, output_format=output_format, aggregation_functions=aggregation_functions)
        return result

    async def count(self,
                    entity_name: str,
                    *,
                    filter_by: List[dict] = None,
                    search_by: List[dict] = None,
                    ) -> int:
        entity = self.get_entity_by_name(entity_name)

        filter_by: List[Filter] = entity.filters_from_str(filter_by)
        search_by: List[Filter] = entity.filters_from_str(search_by)

        result = await self.context.clickhouse_connector.count(entity, filter_by=filter_by, search_by=search_by)
        return result

    async def add(self, entity_name: str, values: Union[dict, List[dict]]) -> None:
        values = to_list(values)

        entity = self.get_entity_by_name(entity_name)

        grouped_values = defaultdict(list)
        for val in values:
            try:
                prepared_val = await entity.constructor(**val)
                insert_column_names = tuple(prepared_val.to_insert_columns())
                grouped_values[insert_column_names].append(prepared_val)
            except (ValueError, TypeError) as e:
                logger.error(f'failed to construct entity: {entity} from {val}, because: {repr(e)}')

        for data_to_insert in grouped_values.values():
            await self.context.clickhouse_connector.insert(data_to_insert)

==> clickhouse_tools_abm-2.0.54563.tar.gz: clickhouse_tools_abm-2.0.54563/clickhouse_tools/clickhouse_interface.py <==
import logging
from collections import defaultdict
from dataclasses import dataclass
from typing import List, Dict, Type, Union, Optional
from async_tools import AsyncOnStart
from dict_caster.extras import to_list

from .elements.aggregation_function import AggregationFunction
from .elements.filter_by import Filter
from .elements.order import Order
from .elements.column import Column
from .exceptions import WrongArgumentsException
from .elements.table import Table, OrderRepresentations, ColumnsRepresentations, ALL_COLUMNS
from .elements.output_format import OutputFormat
from .clickhouse_connector import ClickHouseConnector


logger = logging.getLogger(__file__)


class ClickHouseInterface(AsyncOnStart):
    @dataclass
    class Context:
        clickhouse_connector: ClickHouseConnector

    def __init__(self, context: Context):
        self.context = context
        self._name_to_entity: Dict[str, Type[Table]] = {}

    async def _on_start(self) -> None:
        for table in self._name_to_entity.values():
            await self.context.clickhouse_connector.create_table(table)

    def register_entity(self, entity: Type[Table]) -> None:
        self._name_to_entity[entity.__table_name__] = entity
        logger.info(f'{entity} registered')

    def get_entity_by_name(self, entity_name: str) -> Type[Table]:
        if entity := self._name_to_entity.get(entity_name):
            return entity
        raise WrongArgumentsException(f'Unknown entity: {entity_name}')

    async def get(self,
                  entity_name: str,
                  *,
                  columns: ColumnsRepresentations = ALL_COLUMNS,
                  filter_by: Optional[List[dict]] = None,
                  search_by: Optional[List[dict]] = None,
                  order_by: OrderRepresentations = None,
                  group_by: Optional[List[str]] = None,
                  offset: int = 0,
                  limit: int = 100,
                  output_format: str = OutputFormat.Dict.value,
                  aggregation_functions: Optional[List[dict]] = None
                  ) -> list:

        entity = self.get_entity_by_name(entity_name)

        columns: List[Column] = entity.columns_from_str(columns)
        filter_by: List[Filter] = entity.filters_from_str(filter_by)
        search_by: List[Filter] = entity.filters_from_str(search_by)
        order_by: List[Order] = entity.orders_from_str(order_by)
        group_by: List[Column] = entity.groups_from_str(group_by)
        output_format: OutputFormat = entity.output_format_from_str(output_format)
        aggregation_functions: List[AggregationFunction] = entity.aggregation_functions_from_str(aggregation_functions)

        result = await self.context.clickhouse_connector.select(
            entity, columns=columns, filter_by=filter_by, search_by=search_by, order_by=order_by, group_by=group_by,
            offset=offset, limit=limit, output_format=output_format, aggregation_functions=aggregation_functions)
        return result

    async def count(self,
                    entity_name: str,
                    *,
                    filter_by: List[dict] = None,
                    search_by: List[dict] = None,
                    ) -> int:
        entity = self.get_entity_by_name(entity_name)

        filter_by: List[Filter] = entity.filters_from_str(filter_by)
        search_by: List[Filter] = entity.filters_from_str(search_by)

        result = await self.context.clickhouse_connector.count(entity, filter_by=filter_by, search_by=search_by)
        return result

    async def add(self, entity_name: str, values: Union[dict, List[dict]]) -> None:
        values = to_list(values)

        entity = self.get_entity_by_name(entity_name)

        grouped_values = defaultdict(list)
        for val in values:
            try:
                prepared_val = await entity.constructor(**val)
                insert_column_names = tuple(prepared_val.to_insert_columns())
                grouped_values[insert_column_names].append(prepared_val)
            except (ValueError, TypeError) as e:
                logger.error(f'failed to construct entity: {entity} from {val}, because: {repr(e)}')

        for data_to_insert in grouped_values.values():
            await self.context.clickhouse_connector.insert(data_to_insert)
